@@ -149,4 +149,14 @@ impl<VM: VMBinding> DerefMut for MyGCProcessEdges<VM> {
         &mut self.base
     }
 }
-// ANCHOR_END: deref
\ No newline at end of file
+// ANCHOR_END: deref
+
+// ANCHOR: workcontext
+pub struct MyGCWorkContext<VM: VMBinding>(std::marker::PhantomData<VM>);
+impl<VM: VMBinding> crate::scheduler::GCWorkContext for MyGCWorkContext<VM> {
+    type VM = VM;
+    type PlanType = MyGC<VM>;
+    type CopyContextType = MyGCCopyContext<VM>;
+    type ProcessEdgesWorkType = MyGCProcessEdges<VM>;
+}
+// ANCHOR_END: workcontext
