@@ -9,15 +9,16 @@ use crate::MMTK;
 use std::ops::{Deref, DerefMut};
 
 /// Process edges for a nursery GC. A generatinoal plan should use this type for a nursery GC.
-pub struct GenNurseryProcessEdges<VM: VMBinding, C: CopyContext + GCWorkerLocal> {
+pub struct GenNurseryProcessEdges<VM: VMBinding, C: CopyContext<VM = VM> + GCWorkerLocal> {
     gen: &'static Gen<VM>,
     base: ProcessEdgesBase<GenNurseryProcessEdges<VM, C>>,
 }
 
-impl<VM: VMBinding, C: CopyContext + GCWorkerLocal> ProcessEdgesWork
+impl<VM: VMBinding, C: CopyContext<VM = VM> + GCWorkerLocal> ProcessEdgesWork
     for GenNurseryProcessEdges<VM, C>
 {
     type VM = VM;
+
     fn new(edges: Vec<Address>, roots: bool, mmtk: &'static MMTK<VM>) -> Self {
         let base = ProcessEdgesBase::new(edges, roots, mmtk);
         let gen = base.plan().generational();
@@ -40,14 +41,18 @@ impl<VM: VMBinding, C: CopyContext + GCWorkerLocal> ProcessEdgesWork
     }
 }
 
-impl<VM: VMBinding, C: CopyContext + GCWorkerLocal> Deref for GenNurseryProcessEdges<VM, C> {
+impl<VM: VMBinding, C: CopyContext<VM = VM> + GCWorkerLocal> Deref
+    for GenNurseryProcessEdges<VM, C>
+{
     type Target = ProcessEdgesBase<Self>;
     fn deref(&self) -> &Self::Target {
         &self.base
     }
 }
 
-impl<VM: VMBinding, C: CopyContext + GCWorkerLocal> DerefMut for GenNurseryProcessEdges<VM, C> {
+impl<VM: VMBinding, C: CopyContext<VM = VM> + GCWorkerLocal> DerefMut
+    for GenNurseryProcessEdges<VM, C>
+{
     fn deref_mut(&mut self) -> &mut Self::Target {
         &mut self.base
     }
