@@ -9,7 +9,6 @@ use crate::plan::Mutator;
 use crate::policy::immortalspace::ImmortalSpace;
 use crate::policy::largeobjectspace::LargeObjectSpace;
 use crate::policy::space::Space;
-use crate::scheduler::gc_work::ProcessEdgesWork;
 use crate::scheduler::*;
 use crate::util::alloc::allocators::AllocatorSelector;
 #[cfg(feature = "analysis")]
@@ -136,6 +135,9 @@ pub fn create_mutator<VM: VMBinding>(
         PlanSelector::PageProtect => {
             crate::plan::pageprotect::mutator::create_pp_mutator(tls, &*mmtk.plan)
         }
+        PlanSelector::MarkCompact => {
+            crate::plan::markcompact::mutator::create_markcompact_mutator(tls, &*mmtk.plan)
+        }
     })
 }
 
@@ -166,6 +168,9 @@ pub fn create_plan<VM: VMBinding>(
         PlanSelector::PageProtect => Box::new(crate::plan::pageprotect::PageProtect::new(
             vm_map, mmapper, options,
         )),
+        PlanSelector::MarkCompact => Box::new(crate::plan::markcompact::MarkCompact::new(
+            vm_map, mmapper, options,
+        )),
     }
 }
 
@@ -382,7 +387,6 @@ pub struct BasePlan<VM: VMBinding> {
     pub trigger_gc_when_heap_is_full: AtomicBool,
     pub gc_status: Mutex<GcStatus>,
     pub last_stress_pages: AtomicUsize,
-    pub stacks_prepared: AtomicBool,
     pub emergency_collection: AtomicBool,
     pub user_triggered_collection: AtomicBool,
     pub internal_triggered_collection: AtomicBool,
@@ -401,8 +405,10 @@ pub struct BasePlan<VM: VMBinding> {
     pub heap: HeapMeta,
     #[cfg(feature = "sanity")]
     pub inside_sanity: AtomicBool,
-    // A counter for per-mutator stack scanning
-    pub scanned_stacks: AtomicUsize,
+    /// A counter for per-mutator stack scanning
+    scanned_stacks: AtomicUsize,
+    /// Have we scanned all the stacks?
+    stacks_prepared: AtomicBool,
     pub mutator_iterator_lock: Mutex<()>,
     // A counter that keeps tracks of the number of bytes allocated since last stress test
     allocation_bytes: AtomicUsize,
@@ -712,10 +718,37 @@ impl<VM: VMBinding> BasePlan<VM> {
         }
     }
 
+    /// Are the stacks scanned?
     pub fn stacks_prepared(&self) -> bool {
         self.stacks_prepared.load(Ordering::SeqCst)
     }
 
+    /// Prepare for stack scanning. This is usually used with `inform_stack_scanned()`.
+    /// This should be called before doing stack scanning.
+    pub fn prepare_for_stack_scanning(&self) {
+        self.scanned_stacks.store(0, Ordering::SeqCst);
+        self.stacks_prepared.store(false, Ordering::SeqCst);
+    }
+
+    /// Inform that 1 stack has been scanned. The argument `n_mutators` indicates the
+    /// total stacks we should scan. This method returns true if the number of scanned
+    /// stacks equals the total mutator count. Otherwise it returns false. This method
+    /// is thread safe and we guarantee only one thread will return true.
+    pub fn inform_stack_scanned(&self, n_mutators: usize) -> bool {
+        let old = self.scanned_stacks.fetch_add(1, Ordering::SeqCst);
+        debug_assert!(
+            old < n_mutators,
+            "The number of scanned stacks ({}) is more than the number of mutators ({})",
+            old,
+            n_mutators
+        );
+        let scanning_done = old + 1 == n_mutators;
+        if scanning_done {
+            self.stacks_prepared.store(true, Ordering::SeqCst);
+        }
+        scanning_done
+    }
+
     pub fn gc_in_progress(&self) -> bool {
         *self.gc_status.lock().unwrap() != GcStatus::NotInGC
     }
@@ -773,6 +806,12 @@ impl<VM: VMBinding> BasePlan<VM> {
             || self.options.analysis_factor != DEFAULT_STRESS_FACTOR
     }
 
+    /// Check if we should do precise stress test. If so, we need to check for stress GCs for every allocation.
+    /// Otherwise, we only check in the allocation slow path.
+    pub fn is_precise_stress(&self) -> bool {
+        self.options.precise_stress
+    }
+
     /// Check if we should do a stress GC now. If GC is initialized and the allocation bytes exceeds
     /// the stress factor, we should do a stress GC.
     pub fn should_do_stress_gc(&self) -> bool {
@@ -918,61 +957,6 @@ impl<VM: VMBinding> CommonPlan<VM> {
         self.base.release(tls, full_heap)
     }
 
-    /// Schedule all the common work packets
-    pub fn schedule_common<
-        P: Plan<VM = VM>,
-        E: ProcessEdgesWork<VM = VM>,
-        C: CopyContext<VM = VM> + GCWorkerLocal,
-    >(
-        &self,
-        plan: &'static P,
-        constraints: &'static PlanConstraints,
-        scheduler: &GCWorkScheduler<VM>,
-    ) {
-        use crate::scheduler::gc_work::*;
-
-        // Stop & scan mutators (mutator scanning can happen before STW)
-        scheduler.work_buckets[WorkBucketStage::Unconstrained].add(StopMutators::<E>::new());
-
-        // Prepare global/collectors/mutators
-        scheduler.work_buckets[WorkBucketStage::Prepare].add(Prepare::<P, C>::new(plan));
-
-        // VM-specific weak ref processing
-        scheduler.work_buckets[WorkBucketStage::RefClosure].add(ProcessWeakRefs::<E>::new());
-
-        // Release global/collectors/mutators
-        scheduler.work_buckets[WorkBucketStage::Release].add(Release::<P, C>::new(plan));
-
-        // Analysis GC work
-        #[cfg(feature = "analysis")]
-        {
-            use crate::util::analysis::GcHookWork;
-            scheduler.work_buckets[WorkBucketStage::Unconstrained].add(GcHookWork);
-        }
-
-        // Sanity
-        #[cfg(feature = "sanity")]
-        {
-            use crate::util::sanity::sanity_checker::ScheduleSanityGC;
-            scheduler.work_buckets[WorkBucketStage::Final].add(ScheduleSanityGC::<P, C>::new(plan));
-        }
-
-        // Finalization
-        if !self.base.options.no_finalizer {
-            use crate::util::finalizable_processor::{Finalization, ForwardFinalization};
-            // finalization
-            scheduler.work_buckets[WorkBucketStage::RefClosure].add(Finalization::<E>::new());
-            // forward refs
-            if constraints.needs_forward_after_liveness {
-                scheduler.work_buckets[WorkBucketStage::RefForwarding]
-                    .add(ForwardFinalization::<E>::new());
-            }
-        }
-
-        // Set EndOfGC to run at the end
-        scheduler.set_finalizer(Some(EndOfGC));
-    }
-
     pub fn stacks_prepared(&self) -> bool {
         self.base.stacks_prepared()
     }
