@@ -1,14 +1,16 @@
 use super::MarkSweep;
 use crate::plan::barriers::NoBarrier;
+use crate::plan::mutator_context::create_allocator_mapping;
+use crate::plan::mutator_context::create_space_mapping;
 use crate::plan::mutator_context::Mutator;
 use crate::plan::mutator_context::MutatorConfig;
-use crate::plan::AllocationSemantics as AllocationType;
+use crate::plan::mutator_context::ReservedAllocators;
+use crate::plan::AllocationSemantics;
 use crate::util::alloc::allocators::AllocatorSelector;
 use crate::util::alloc::allocators::Allocators;
 use crate::util::{VMMutatorThread, VMWorkerThread};
 use crate::vm::VMBinding;
 use crate::Plan;
-use enum_map::enum_map;
 use enum_map::EnumMap;
 
 pub fn ms_mutator_prepare<VM: VMBinding>(_mutator: &mut Mutator<VM>, _tls: VMWorkerThread) {
@@ -19,11 +21,16 @@ pub fn ms_mutator_release<VM: VMBinding>(_mutator: &mut Mutator<VM>, _tls: VMWor
     // Do nothing
 }
 
+const RESERVED_ALLOCATORS: ReservedAllocators = ReservedAllocators {
+    n_malloc: 1,
+    ..ReservedAllocators::DEFAULT
+};
+
 lazy_static! {
-    pub static ref ALLOCATOR_MAPPING: EnumMap<AllocationType, AllocatorSelector> = enum_map! {
-        AllocationType::Default => AllocatorSelector::Malloc(0),
-        AllocationType::Immortal | AllocationType::Code | AllocationType::LargeCode | AllocationType::ReadOnly => AllocatorSelector::BumpPointer(0),
-        AllocationType::Los => AllocatorSelector::LargeObject(0),
+    pub static ref ALLOCATOR_MAPPING: EnumMap<AllocationSemantics, AllocatorSelector> = {
+        let mut map = create_allocator_mapping(RESERVED_ALLOCATORS, true);
+        map[AllocationSemantics::Default] = AllocatorSelector::Malloc(0);
+        map
     };
 }
 
@@ -34,14 +41,11 @@ pub fn create_ms_mutator<VM: VMBinding>(
     let ms = plan.downcast_ref::<MarkSweep<VM>>().unwrap();
     let config = MutatorConfig {
         allocator_mapping: &*ALLOCATOR_MAPPING,
-        space_mapping: box vec![
-            (AllocatorSelector::Malloc(0), ms.ms_space()),
-            (
-                AllocatorSelector::BumpPointer(0),
-                ms.common().get_immortal(),
-            ),
-            (AllocatorSelector::LargeObject(0), ms.common().get_los()),
-        ],
+        space_mapping: box {
+            let mut vec = create_space_mapping(RESERVED_ALLOCATORS, true, plan);
+            vec.push((AllocatorSelector::Malloc(0), ms.ms_space()));
+            vec
+        },
         prepare_func: &ms_mutator_prepare,
         release_func: &ms_mutator_release,
     };
