@@ -18,6 +18,7 @@ pub struct PPProcessEdges<VM: VMBinding> {
 impl<VM: VMBinding> ProcessEdgesWork for PPProcessEdges<VM> {
     const OVERWRITE_REFERENCE: bool = false;
     type VM = VM;
+
     fn new(edges: Vec<Address>, roots: bool, mmtk: &'static MMTK<VM>) -> Self {
         let base = ProcessEdgesBase::new(edges, roots, mmtk);
         let plan = base.plan().downcast_ref::<PageProtect<VM>>().unwrap();
@@ -56,3 +57,11 @@ impl<VM: VMBinding> DerefMut for PPProcessEdges<VM> {
         &mut self.base
     }
 }
+
+pub struct PPGCWorkContext<VM: VMBinding>(std::marker::PhantomData<VM>);
+impl<VM: VMBinding> crate::scheduler::GCWorkContext for PPGCWorkContext<VM> {
+    type VM = VM;
+    type PlanType = PageProtect<VM>;
+    type CopyContextType = NoCopy<VM>;
+    type ProcessEdgesWorkType = PPProcessEdges<VM>;
+}
