@@ -1,7 +1,7 @@
 use super::block::{Block, BlockState};
 use super::defrag::Histogram;
 use super::immixspace::ImmixSpace;
-use crate::util::metadata::side_metadata::{self, SideMetadataOffset, SideMetadataSpec};
+use crate::util::metadata::side_metadata::{self, SideMetadataSpec};
 use crate::{
     scheduler::*,
     util::{heap::layout::vm_layout_constants::LOG_BYTES_IN_CHUNK, Address},
@@ -134,12 +134,8 @@ pub struct ChunkMap {
 
 impl ChunkMap {
     /// Chunk alloc table
-    pub const ALLOC_TABLE: SideMetadataSpec = SideMetadataSpec {
-        is_global: false,
-        offset: SideMetadataOffset::layout_after(&Block::MARK_TABLE),
-        log_num_of_bits: 3,
-        log_min_obj_size: Chunk::LOG_BYTES,
-    };
+    pub const ALLOC_TABLE: SideMetadataSpec =
+        crate::util::metadata::side_metadata::spec_defs::IX_CHUNK_MARK;
 
     pub fn new() -> Self {
         Self {
