@@ -17,6 +17,9 @@ pub use malloc_allocator::MallocAllocator;
 pub mod immix_allocator;
 pub use self::immix_allocator::ImmixAllocator;
 
+mod markcompact_allocator;
+pub use markcompact_allocator::MarkCompactAllocator;
+
 pub(crate) mod dump_linear_scan;
 pub(crate) mod embedded_meta_data;
 pub(crate) mod linear_scan;
